
#include <microM.h>
#include <Servo.h>
#include <EEPROM.h>
#include "Constants.h"
#include "IOpins.h"


// define global variables
byte mode;                                         // 0 = Stop, 1 = Play, 2 = Calibration
byte count;                                        // counts total number of moves made. 9 = stalemate
byte robotcount;                                   // counts robot moves - used in advanced strategy
byte level=1;                                      // 0 = Beginner, 1 = Novice, 2 = Hard, 3 = Expert
byte move=1;                                       // 0 = Robot, 1 = Player
byte first;                                        // 0 = robots first move, 1 = robots first move has been made
byte robotmove;                                    // grid position chosen by the robot to move to
byte firstmove;                                    // memory of first move used by advanced strategy
byte IRC;                                          // Infra Red Command from TV remote
byte NC;                                           // number command
byte reset=2;                                      // 0 = board has been cleared, 1 = game finished, balls in place, 2 = Ready
byte grid[10];                                     // contents of the game grid - note: index 0 is the content of the gripper
                                                   // 0 = No ball, 1 = Robot ball, 2 = Player ball, 3 = don't know

int shoulderpos[14];                               // array stores the shoulder servo positions for the grid, pickup and dropoff locations
int elbowpos[14];                                  // array stores the elbow    servo positions for the grid, pickup and dropoff locations

                                                   // index   0 = current position                                  
                                                   // index 1-9 = grid positions 1-9                        10            13 
                                                   // index  10 = pickup  position of robot  ball chute     ||    1 2 3   ||
                                                   // index  11 = dropoff position of robot  ball chute     ||    4 5 6   ||
                                                   // index  12 = pickup  position of player ball chute     ||    7 8 9   ||
                                                   // index  13 = dropoff position of player ball chute     11            12

Servo Shoulder;                                    // define Shoulder as a servo
Servo Elbow;                                       // define Elbow    as a servo
Servo Gripper;                                     // define Gripper  as a servo
Servo Pins;                                        // define Pins     as a servo



void setup()
{
  Shoulder.attach(Shoulderpin,600,2400);           // attach Shoulder servo to  D9
  Elbow   .attach(Elbowpin,600,2400);              // attach Elbow    servo to D10
  Gripper .attach(Gripperpin,600,2400);            // attach Gripper  servo to D11
  Pins    .attach(Pinspin,600,2400);               // attach Pins     servo to D12
  
  Pins    .writeMicroseconds(Pinslower);           // lower pins
  delay(400);                                      // allow small delay between initialization of each servo to reduce power surge
  Gripper .writeMicroseconds(Gripupper);           // raise gripper so arm can move freely
  delay(400);
  Elbow   .writeMicroseconds(1500);                // initialize elbow    to center position
  delay(400);
  Shoulder.writeMicroseconds(1500);                // initialize shoulder to center position
  delay(400);
  
  eepromRead();                                    // loads servo positions from EEPROM memory
  for(int i=0;i<14;i++)                            // check for uncalibrated positions and default to 1500uS
  {
    if(shoulderpos[i]<600 || shoulderpos[i]>2400) shoulderpos[i]=1500;
    if(elbowpos[i]<600 || elbowpos[i]>2400) elbowpos[i]=1500;
  }
  microM.Setup();                                  // initialize timer 2 for use with microM.ircommand
  
  digitalWrite(4,1);                               // enable pullup resistor on digital pin 4 (IR receiver)
  
  pinMode(5,OUTPUT);                               // RED   LED
  pinMode(6,OUTPUT);                               // GREEN LED
  pinMode(7,OUTPUT);                               // BLUE  LED
  pinMode(8,OUTPUT);                               // GND   LED
  
  Serial.begin(9600);
  Serial.println("");Serial.println("");
  Serial.println("Tic Tac Toe robot online...");
}


void loop()
{//================================================== Mode Selection ==========================================
  
                                                   // Mode 0: Reset grid and begin new game
                                                   // Mode 1: Play Game
                                                   // Mode 2: Calibration
                                                   // Mode 3: Game Over - wait for rest button
  
  digitalWrite(5,(mode==2));                       // turn on RED   LED during calibration mode
  digitalWrite(6,(mode==1));                       // turn on GREEN LED during play mode
  digitalWrite(7,(mode==0));                       // turn on BLUE  LED during board reset proceedure
  
  IRC=microM.ircommand;                            // read IR receiver
  microM.ircommand=0;                              // reset receiver value to prevent repeating the command
  delay(500);                                      // debounce timer - most TV remotes repeat the signal at least once
  
                                                   // not all remotes return the values 1-9 for keys 1-9 and 10 for key 0
  NC=0;                                            // NC is used so that any keys can be converted to the Numeric Commands 1-10
  if(IRC==Onekey)   NC=1;
  if(IRC==Twokey)   NC=2;
  if(IRC==Threekey) NC=3;
  if(IRC==Fourkey)  NC=4;
  if(IRC==Fivekey)  NC=5;
  if(IRC==Sixkey)   NC=6;
  if(IRC==Sevenkey) NC==7;
  if(IRC==Eightkey) NC==8;
  if(IRC==Ninekey)  NC==9;
  if(IRC==Zerokey)  NC=10;
  
  if(IRC!=0) Serial.println(IRC,DEC);              
  
  if(IRC==Modekey)                                 // Orange power button pressed
  {
    mode+=1;                                       // Toggle between calibrate and reset board
    if(mode>2)
    {
      mode=0;
      Gripper .writeMicroseconds(Gripupper);       // raise gripper so arm can move freely
      delay(400);
    }
    
    Serial.print("Mode:");Serial.println(mode,DEC); 
  }
  
  if(mode==2) 
  {
    Calibrate();                                   // Manual calibrations of grid positions;  
    return;
  }
  
  if(IRC==Quitkey)                                 // Green button pressed
  {
    mode=0;                                        // Quit and start new game
    reset=1;
    Serial.print("Mode:");Serial.println(mode,DEC);
  }
  
  if(mode==3) return;
  
  if(mode==0)
  {
    if(reset==0) mode=1;                           // board reset - begin new game 
    if(reset==1) 
    {
      Serial.println("Clearing Board for new game");
      ClearBoard();                                // Game finished - clear board
      reset=2;
    }
    if(reset==2)
    {
      Serial.print("Difficulty level = ");Serial.println(level,DEC);
      Serial.println("");
      if(move==1) Serial.println("Waiting for player to make first move");
      DisplayGrid();
      reset=0;
      count=0;
      robotcount=0;
    }
  }
  
  if(mode!=1) return;
  if(move==1 && IRC==0) return;
  
  //================================================= Play Game =====================================
  
  if(IRC==Leveldeckey && level>0) level--;
  if(IRC==Levelinckey && level<3) level++;
  if(IRC==Levelinckey || IRC==Leveldeckey)
  {
    Serial.print("Difficulty level = ");Serial.println(level,DEC);
    return;
  }
  Serial.println(count,DEC);
  
  if(move==1)//-------------------------------------- PLAYER MOVES ----------------------------------
  {
    if(NC>0 && grid[NC]==0)                        // only empty grid positions 1-9 are valid moves
    {
      Serial.print("Player selects grid position: "); Serial.println(IRC,DEC);
      
      MoveArm(12);                                 // goto player ball pickup location
      Gripper .writeMicroseconds(Griplower);       // lower gripper to pick up players ball
      delay(2000);                                 // allow time for servo to move and grip ball
      grid[0]=2;                                   // gripper now holding players ball
      Gripper .writeMicroseconds(Gripcarry);       // pick up players ball
      delay(300);
      MoveArm(NC);                                // move arm to position selected by player
      Gripper .writeMicroseconds(Gripupper);       // raise gripper to drop players ball
      delay(300);
      grid[0]=0;                                   // gripper is no longer holding a ball
      grid[NC]=2;                                 // players ball in grid position IRC
      move=0;                                      // now robots turn to move
      count++;                                     // if count=9 then no more moves can be made
    }
    else
    {
      Serial.println("Invalid move");
      return;
    }
    
    if(CheckForWin()) mode=3;                      // is the game over?
  }
  else //-------------------------------------------- ROBOT MOVES -------------------------------------
  {
    robotmove=0;                                   // forget any previous move
    if(first==0 || level==0) RandomMove();         // robots first move and beginner level moves are random
    if(level>1 && robotmove==0) BlockPlayer();     // after first move, hard and expert levels checks if it needs to block you every move
    if(random(5)>2 && robotmove==0) BlockPlayer(); // after first move, novice level checks if it needs to block you 3 out of 5 moves
    if(level>2 && robotcount==1) SecondMove();     // in expert level robot plans second move 
    if(robotmove==0 && level>2 && robotcount==2 && grid[5]==0) robotmove=5;  // completes expert strategy
    if(robotmove==0) MakeLine();                   // if move is not random or block then try to complete a line
    if(robotmove==0) RandomMove();                 // no lines to complete, make a random move
    
    Serial.print("Robot selects grid position: "); Serial.println(robotmove,DEC);
    MoveArm(10);                                   // goto robot ball pickup location
    Gripper .writeMicroseconds(Griplower);         // lower gripper to pick up players ball
    delay(2000);                                   // allow time for servo to move and grip ball
    grid[0]=1;                                     // gripper now holding robots ball
    Gripper .writeMicroseconds(Gripcarry);         // pick up robots ball
    delay(300);
    MoveArm(robotmove);                            // move arm to position selected by robot
    Gripper .writeMicroseconds(Gripupper);         // raise gripper to drop robots ball
    delay(300);
    grid[0]=0;                                     // gripper is no longer holding a ball
    grid[robotmove]=1;                             // robots ball in grid position robotmove
    move=1;                                        // now players turn to move
    count++;                                       // if count=9 then no more moves can be made
    robotcount++;
  } 
  
  DisplayGrid();
  if(count>8)
  {
    Serial.println("No more moves left!");
    mode=0;
    reset=1;
    return;
  }
  
  if(CheckForWin()) mode=3;                        // is the game over?
}



